#include <cellrenderercolor.hpp>
#include <gtk/gtkcombobox.h>

#include <iostream>
using namespace std ;

void CellRendererColor_item_changed (CellRendererColor* crColor, Gtk::ComboBox* combo)
{
	Gtk::TreeIter iter = combo->get_active() ;
	crColor->set_current_color (iter) ;

	crColor->set_color_changed (true) ;
	crColor->get_changed_signal().disconnect() ;
}

void CellRendererColor_on_editing_started (
		GtkCellRenderer*, 
		GtkCellEditable* cell_editable, 
		const gchar*,
		void* data
		)
{
	CellRendererColor* pCppSelf = (CellRendererColor*)data;

	if(cell_editable)
	{
		//This is actually ComboBox, because GtkComboBox inherits from GtkCellEditable since GTK+ 2.6.
		//But Gtk::ComboBox does not inherit from Gtk::CellEditable because we could not break ABI.
		//So we will use the C API to get the Gtk::ComboBox:
		GtkComboBox* pCComboBox = GTK_COMBO_BOX(cell_editable);
		Gtk::ComboBox* pComboBox = Glib::wrap(pCComboBox);


		//We don't use this convenience method, because we want more control over the renderer.
		//and CellLayout gives no way to get the renderer back afterwards.
		//(well, maybe set_cell_data_func(), but that's a bit awkward.)
		//pComboBox->pack_start(pCppSelf->m_model_columns.m_col_extra);

		Gtk::CellRendererPixbuf* cell_second = Gtk::manage(new Gtk::CellRendererPixbuf);
		cell_second->property_xalign() = 0.0;
		cell_second->property_xpad() = 10;

		Gtk::CellRendererText* red_cell = Gtk::manage(new Gtk::CellRendererText);
		red_cell->property_visible() = false ;

		Gtk::CellRendererText* green_cell = Gtk::manage(new Gtk::CellRendererText);
		green_cell->property_visible() = false ;

		Gtk::CellRendererText* blue_cell = Gtk::manage(new Gtk::CellRendererText);
		blue_cell->property_visible() = false ;

		//Use the renderer:
		pComboBox->pack_start(*cell_second, false);
		pComboBox->pack_start(*red_cell, false);
		pComboBox->pack_start(*green_cell, false);
		pComboBox->pack_start(*blue_cell, false);
		pComboBox->reorder (*cell_second, 0) ;

		//Make the renderer render the column:
		pComboBox->add_attribute(cell_second->_property_renderable(), 
				pCppSelf->m_columns.m_image);

		pComboBox->add_attribute(red_cell->_property_renderable(), 
				pCppSelf->m_columns.m_red);

		pComboBox->add_attribute(green_cell->_property_renderable(), 
				pCppSelf->m_columns.m_green);

		pComboBox->add_attribute(blue_cell->_property_renderable(), 
				pCppSelf->m_columns.m_blue);

		pCppSelf->conn_changed = pComboBox->signal_changed().connect (
				sigc::bind<CellRendererColor*, Gtk::ComboBox*> (
					sigc::ptr_fun (&CellRendererColor_item_changed), pCppSelf, pComboBox)) ;

	}
	else
	{
		g_warning("CellRendererColor::on_editing_started() cell_editable is null");
	}
}

void CellRendererColor_on_edited (
		GtkCellRenderer* obj, 
		gchar *path,
		gchar *new_text,
		void* data
		)
{
	CellRendererColor* pCppSelf = (CellRendererColor*)data;

	if (pCppSelf)
	{
		g_signal_handler_block (obj, pCppSelf->edited_signal) ;
		pCppSelf->edited(Glib::ustring(path), Glib::ustring(new_text)) ;
		if (pCppSelf->get_color_changed())
		{
	//		pCppSelf->m_colorSignal (Glib::ustring(path), pCppSelf->m_currentColor) ;
			pCppSelf->set_color_changed (false) ;
		}
		g_signal_handler_unblock (obj, pCppSelf->edited_signal) ;
	}
}

void CellRendererColor :: set_current_color (const Gtk::TreeIter& iter)
{
	Gtk::TreeRow row = *iter ;

	float r = row[m_columns.m_red] ;
	float g = row[m_columns.m_green] ;
	float b = row[m_columns.m_blue] ;

	m_currentColor.set_rgb_p (r, g, b) ;
}

void CellRendererColor :: add_color (const Gdk::Color& c, const Glib::ustring& name)
{
	Glib::RefPtr <Gdk::Pixbuf> image = create_from_color (c) ;

	Gtk::TreeModel::Row row = *(m_model->append()) ;
	row[m_columns.m_col1] = name ;
	row[m_columns.m_image] = image ;

	row[m_columns.m_red]   = c.get_red_p() ;
	row[m_columns.m_green] = c.get_green_p() ;
	row[m_columns.m_blue]  = c.get_blue_p() ;
}

void CellRendererColor :: remove_all_items (void)
{
	if (m_model)
		m_model->clear() ;
}

CellRendererColor :: CellRendererColor (Gtk::Widget* parent)
	: Glib::ObjectBase(0), m_parent (parent)
{
	m_model = Gtk::ListStore::create (m_columns) ;
	m_colorChanged = false ;

	property_text_column() = 0 ;
	property_editable() = true ;
	property_model() = m_model ;

	g_signal_connect (
			gobj(), 
			"editing_started", 
			G_CALLBACK(&CellRendererColor_on_editing_started), 
			this
			);

	edited_signal = g_signal_connect (
			gobj(), 
			"edited", 
			G_CALLBACK(&CellRendererColor_on_edited), 
			this
			);
}

CellRendererColor :: ~CellRendererColor ()
{
}

Glib::RefPtr<Gdk::Pixbuf> CellRendererColor :: create_from_color (const Gdk::Color& c)
{
	int width (12), height (12) ;

	Glib::RefPtr<Gdk::Pixmap> pixmap = Gdk::Pixmap::create (
			m_parent->get_window(),
			width, 
			height
			) ;

	Cairo::RefPtr<Cairo::Context> context = pixmap->create_cairo_context() ;

	context->set_source_rgb (c.get_red_p(), c.get_green_p(), c.get_blue_p()) ;
	context->rectangle (0, 0, width, height) ;
	context->paint() ;

	Glib::RefPtr<Gdk::Image> image = pixmap->get_image(0, 0, width, height) ;
	Glib::RefPtr<Gdk::Pixbuf> pixbuf = Gdk::Pixbuf::create (
			image, 0, 0, width, height) ;

	return pixbuf ;
}

